#!/opt/VRTSperl/bin/perl -w
#------------------------------------------------------------------
# $Copyright: Copyright (c) 2017 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#
# $Header: /project/isv-agent/VProAgents/WebSphereMQ/MQ_update.pl,v 1.10 2015/08/25 08:20:47 mrampura Exp $
#
# $sAuthor: $
#
# $Date: 2015/08/25 08:20:47 $
# $Revision: 1.10 $
#------------------------------------------------------------------

use strict;
use POSIX;
use Getopt::Std;
use Fcntl;
use File::Path qw ( mkpath );

use constant HASYS     => "/opt/VRTSvcs/bin/hasys";
use constant HARES     => "/opt/VRTSvcs/bin/hares";
use constant HATYPE    => "/opt/VRTSvcs/bin/hatype";
use constant HACONF    => "/opt/VRTSvcs/bin/haconf";
use constant HACF      => "/opt/VRTSvcs/bin/hacf";
use constant HACLUS    => "/opt/VRTSvcs/bin/haclus";
use constant HAATTR    => "/opt/VRTSvcs/bin/haattr";
use constant MAINCFLOC => "/etc/VRTSvcs/conf/config";
use constant VCSLOGDIR => "/var/VRTSvcs/log";

sub usage {
  warn << "EOF";

  Update:- Update the WebSphereMQ6 Type with WebSphereMQ type.
  
  Usage:
  
  $0 

  -h  --help      Help (this message)
EOF

  exit 1;
}

my ( $sCommand, $sCmdOut, $iExitCode, $sMainCmdFile, $sAttribute, $sRestoreMsg,
     $sOptStr, $SystemList, $sRmPkg, $sTypeFile, $sRes, $sLine, $bSuccessFlag, 
     $sTempCmdFile, $sLogDir, $sOldArgList, $sNewArgList, $iPid, $sKey, @lsRes, @lsArgList, 
     @lsDepList, %hsOpts, %hsResAttr, $sNewTypeFile, $bVCSStatus ) = ();

my %hsSupported = ( '--help'    => '-h' );
my %hsOverriddenResAttr = ();

#-----------------------
# Collect command line..
#-----------------------
$sOptStr = 'h:o';

grep {                      # Filter..
  $hsSupported { $_ }    && # Defined..
  s/$_/$hsSupported{$_}/ || # Then substitute..
  1                         # Else, continue..
} @ARGV;                    # From what's been passed..

&getopts ( $sOptStr, \%hsOpts ) || &usage ();

#------------
# Show Usage 
#------------
if ( $hsOpts { h } ) {
  &usage();
}

my $sOldType = "WebSphereMQ6";
my $sNewType = "WebSphereMQ";

my $bFlagBothTypes = 0;
my $FlagOldTypeLatest = 0;

# Show info and confirm for update 
# print "\nThis script will add WebSphereMQ Agent Type and reconfigure all 
#         the existing resources of WebSphereMQ6 Type  with  WebSphereMQ Type.
#         This will also remove the WebSphereMQ6 Agent Type.\n";
#-------------------------------------------
# Create the Log Dir, If not already present
#-------------------------------------------
$sLogDir = VCSLOGDIR . "/Update${sNewType}Type_$$";

if ( -e $sLogDir ) {
  $sCommand = "/bin/rm -rf $sLogDir > /dev/null 2>&1";
  system ( $sCommand );
  if( $iExitCode = $? >> 8 ) {
    warn "Failed to remove existing log Directory [$sLogDir]\n";
    exit 1;
  }  
}

#-------------------
# Create the Log Dir
#-------------------
if ( ! mkpath ( $sLogDir, 0, 0700 ) ) {
  warn "Failed to make log directory [$sLogDir]: $!\n";
  exit 1;   
}

#--------------------------------------
# Open the log file for printing output
#--------------------------------------

sysopen ( LOG, "$sLogDir/MQ_update.log",
          O_WRONLY|O_SYNC|O_CREAT|O_TRUNC, 0640 ) ||
die "Failed to create and open file  [$sLogDir/MQ_update.log]: $!\n";
#print "Log file is created at: $sLogDir/MQ_update.log\n";

#------------------------
# Create Log Header 
#------------------------

my @lsOsDetails = uname ();
my $sTime = strftime '%Y/%m/%d %H:%M:%S', localtime; 

my $sHeader = "#
# Log Name:     $sLogDir/MQ_update.log
# System:       $lsOsDetails[1]
# SysInfo:      $lsOsDetails[0]:$lsOsDetails[1],$lsOsDetails[3],$lsOsDetails[2],$lsOsDetails[-1]
# Created:      $sTime
# \n\n";
$sHeader =~ s/SunOS/Solaris/g;

print LOG $sHeader;

#-------------------------------
# Check if VCS is running or not 
#-------------------------------
$bVCSStatus = &getVCSStatus();

#---------------------------------------
# Proceed only if new type is installed
#---------------------------------------
if ( -e ( $sNewTypeFile = "/etc/VRTSagents/ha/conf/${sNewType}/${sNewType}Types.cf"   ) ||  # For Linux,HPUX,AIX
     -e ( $sNewTypeFile = "/etc/VRTSagents/ha/conf/${sNewType}/${sNewType}Types51.cf" ) ||  # For Solaris
     -e ( $sNewTypeFile = "/etc/VRTSagents/ha/conf/${sNewType}/${sNewType}Types50.cf" ) ||
     -e ( $sNewTypeFile = "/etc/VRTSvcs/conf/sample_${sNewType}/${sNewType}Types.cf" ) ) {  
}
else {
	print LOG "[Failed] $sNewType Agent is not installed.\n";
	print "For more details check the log file [$sLogDir/MQ_update.log]\n";
	close ( LOG ) || die "Failed to close [$sLogDir/MQ_update.log] after read: $!\n";
	exit 0;
}

#---------------------------
# Take the backup of main.cf
#---------------------------
$sCommand = "/bin/cp -f /etc/VRTSvcs/conf/config/main.cf $sLogDir/main.cf.before_update > /dev/null 2>&1";
system ( $sCommand );

if ( $iExitCode = $? >> 8 ) {
  printError ("Failed to backup the current main.cf");
  close ( LOG );
  exit 1;
}
else {
  print LOG "[Success] The backup of main.cf before update is ". 
            "present at [$sLogDir/main.cf.before_update]\n";
}

$sRestoreMsg = "\n\nThe backup of main.cf before the execution of this update script is available 
at [$sLogDir/main.cf.before_update]. You can restore your configuration using this file.\n";

#---------------------------------
# Generate main.cmd from main.cf 
# and place it in LogDir
#---------------------------------
$sCommand = HACF . " -cftocmd " . MAINCFLOC .  " -dest $sLogDir > /dev/null 2>&1";
system ( $sCommand );

if ( $iExitCode = $? >> 8 ) {
  printError ("Failed to create main.cmd from main.cf");
  close ( LOG );
  exit 1;
}
else {
  print LOG "[Success] main.cmd is generated from main.cf\n";
}

if ( $bVCSStatus ) { 			# If VCS is Running.... 
        print LOG "VCS is currently running.\n";

	#-------------------------------
	# Dump the current configuration
	#-------------------------------
	$sCommand = HACONF . " -dump > /dev/null 2>&1";
	system ( $sCommand );

	if ( $iExitCode = $? >> 8 ) {
		printError ("Failed to dump the current configuration in main.cf");
		close ( LOG );
		exit 1;
	}
	else {
		print LOG "[Success] Current configuration is dumped in main.cf\n";
	}

	#---------------------------------
	# Check old type is present or not
	#---------------------------------
	$sCommand =  HATYPE . " -display $sOldType";
	$sCmdOut = `$sCommand 2>&1`;
	if ( $iExitCode = $? >> 8 ) {

		if ( $sCmdOut =~ /Type does not exist/ ) {
			print LOG "[Info] The Agent Type [$sOldType] is not imported in the cluster\n";
			$sCommand =  HATYPE . " -display $sNewType";
			$sCmdOut = `$sCommand 2>&1`;
			$sOldType = $sNewType ;
			$FlagOldTypeLatest = 1;
			if ( $iExitCode = $? >> 8 ) {
				if ( $sCmdOut =~ /Type does not exist/ ) {
					print LOG "[Info] The Agent Type [$sOldType] is not imported in the cluster\n";
					print "For more details check the log file [$sLogDir/MQ_update.log]\n";
					close ( LOG );
					exit 0;
				} else {
                      		print LOG "Check whether VCS is running on the system. If not, start VCS and try running MQ_update.pl script again\n";
				printError("Not able to execute [hatype] command on this node");
                        	close ( LOG );
                        	exit 1;
				}
			}
		} else {
			print LOG "Check whether VCS is running on the system. If not, start VCS and try running MQ_update.pl script again\n";
			printError("Not able to execute [hatype] command on this node");
                        close ( LOG );
			exit 1;
		}
	}  
	else {
		print LOG "[Success] Old Type [$sOldType] exists check passed\n";
	} 

	if ( $FlagOldTypeLatest == 0){
		#-----------------------------------------
		# Confirm New Type is not already imported
		#-----------------------------------------
		$sCommand =  HATYPE . " -display $sNewType";
       		$sCmdOut = `$sCommand 2>&1`;

	        if ( $sCmdOut =~ /Cannot connect to VCS engine/ ){
       		        print LOG "Check whether vcs is running or not. If not start vcs and try the update script again\n";
                  	printError("Not able to execute [hatype] command on this node");
			close ( LOG );
                  	exit 1;
        	} elsif ( $sCmdOut =~ /Type does not exist/ ) {
                  print LOG "[Success] New Type [$sNewType] is not imported\n";
        	} else {
		  	warn "The agent type [$sNewType] is already imported in the cluster\n";
		  	print LOG "[Info] The agent type [$sNewType] is already imported in the cluster\n";
                  	$bFlagBothTypes = 1;
        	}	
	}

	$sCommand = HATYPE . " -value $sOldType ArgList";
	$sCmdOut = `$sCommand 2>&1`;
	if ( $iExitCode = $? >> 8 ) {
		printError ("Not able to get the ArgList of old type");
		close ( LOG );
		exit 1;
	}
	else {
		print LOG "[Success] Get the ArgList of old type\n";
		$sOldArgList = $sCmdOut;
	}
	@lsArgList = ( $sCmdOut =~ /(\S+)/g );

	if ( -e ( $sTypeFile = "/etc/VRTSagents/ha/conf/${sNewType}/${sNewType}Types.cf" )  ||
			-e ( $sTypeFile = "/etc/VRTSagents/ha/conf/${sNewType}/${sNewType}Types51.cf" ) ||
			-e ( $sTypeFile = "/etc/VRTSagents/ha/conf/${sNewType}/${sNewType}Types50.cf" ) ||
			-e ( $sTypeFile = "/etc/VRTSvcs/conf/sample_${sNewType}/${sNewType}Types.cf" ) ) {
		if ( ! open ( TYPEFILDES, $sTypeFile ) ) {
			printError("Failed to open Agent type file [$sTypeFile] for read");
			close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] Agent Type File[$sTypeFile] is opened for read\n";
		}
	}
	else {
		printError("[Failure] Agent Type File for [$sNewType] does not exist, Re-install the latest agent package");
		close ( LOG );
		exit 1;
	}

	while ( $sLine = <TYPEFILDES> ) {
		if ( $sLine =~ m/.*ArgList.*=\s+\{\s+(.*)\}/ ) {
			$sNewArgList = $1 if $1;
		}
		$sNewArgList =~ s/,//g if $sNewArgList;
	}
	close ( TYPEFILDES );

	#my $sConfirm;

	#print "\nThis script will reconfigure the [WebSphereMQ6] type resources to [WebSphereMQ] type resources.\nDo you want to proceed (y/n):";

	#$sConfirm = <>;
	#if ( ! ( $sConfirm =~ /^(y|Y)$/ ) ){
		#close ( LOG );
		#exit 1;
	#}

	#-------------------------------------
	# Obtain the resource list of Old Type
	#-------------------------------------
	$sCommand = HATYPE . " -resources $sOldType";
	$sCmdOut = `$sCommand 2>&1`;
	@lsRes = ( $sCmdOut =~ m/(\S+)\n/g );

	#----------------------------------------
	# Filter Type specific data from main.cmd
	#----------------------------------------
	$sMainCmdFile = "$sLogDir/main.cmd";
	$sTempCmdFile = "$sLogDir/main_temp.cmd";

	if ( ! open ( FILDES, $sMainCmdFile ) ) {
		printError("Failed to open created file [$sMainCmdFile] for read"); 
		close ( LOG );
		exit 1;
	}
	else {
		print LOG "[Success] main.cmd is opened for read\n";
	}

	if ( ! sysopen ( TEMPFILDES, $sTempCmdFile,
				O_WRONLY|O_CREAT|O_TRUNC, 0755 ) ) {
		printError("Failed to create and open file [$sTempCmdFile]: $!");
		close ( LOG );
		exit 1;
	}
	else {
		print LOG "[Success] Temprory File [$sTempCmdFile] is created and opened ".
			"for filtering Type specific data\n";
	}

	print TEMPFILDES "#!/bin/sh\n";
	print TEMPFILDES "PATH=/opt/VRTSvcs/bin:\$PATH\nexport PATH\n";
	$sLine = ();
        my $sFlag_AttrDisabledProcesses = 0;
        my $sFlag_AttrStartOptions = 0;
	my $sFlag_AttrStopOptions = 0;
	my $sFlag_AttrCleanIPC = 0;
	my $sFlag_AttrTransitionWaitLimit = 0;
	my $sFlag_AttrServiceName = 0;
        my $sFlag_AttrSystemDEnvFile = 0;

	while ( $sLine = <FILDES> ) {
		if ( ( ( $sLine =~ m/\b$sOldType\b/g ) && 
					( $sLine =~ m/\b(hatype|haattr)\b/ ) ) &&
				! ( $sLine =~ m/hares/ ) ) {
			if ( $sLine =~ m/\bSourceFile\b/ ) {
				$sLine =~ s/$sOldType/$sNewType/g;
			}
			else {
				$sLine =~ s/\b$sOldType\b/$sNewType/g;
			}

			if ( $sLine =~ m/\bArgList\b/ ) {
				print LOG "Replacing Arglist\n";
				$sLine =~ s/\bArgList.*/ArgList $sNewArgList/g;
			}

                        if ( $sLine =~ m/MonitorListener\s+-boolean\s+0/){
                             $sLine =~ s/MonitorListener\s+-boolean\s+0/MonitorListener -integer 0/;
                        }

                        if ( $sLine =~ m/DisabledProcesses\s+-keylist/){
                             $sFlag_AttrDisabledProcesses = 1;
                        }
			elsif ($sLine =~ m/StartOptions\s+-string/){
			     $sFlag_AttrStartOptions = 1;
			}
			elsif ($sLine =~ m/TransitionWaitLimit\s+-integer/){
			     $sFlag_AttrTransitionWaitLimit = 1;
			}
			elsif ($sLine =~ m/StopOptions\s+-string/){
			     $sFlag_AttrStopOptions = 1;
			}
			elsif ($sLine =~ m/CleanIPC\s+-boolean/){
                             $sFlag_AttrCleanIPC = 1;
                        }
			elsif ($sLine =~ m/ServiceName\s+-string/){
			     $sFlag_AttrServiceName = 1;
			}
                        elsif ($sLine =~ m/SystemDEnvFile\s+-string/){
                            $sFlag_AttrSystemDEnvFile = 1;
                        }

			print TEMPFILDES $sLine;

		} else {
                      if ( $sLine =~ m/hares\s+-override\s+\b(\S+)\b\s+(\S+)/g ){
                           if( grep { /$1/ } @lsRes ) {
                               push(@{$hsOverriddenResAttr{$1}},$2);
                           }
                      }
                } 
	}

	if($sFlag_AttrDisabledProcesses == 0){
	   $sLine = HAATTR . " -add $sNewType DisabledProcesses -keylist\n";
	   print TEMPFILDES $sLine;
	}
        if($sFlag_AttrTransitionWaitLimit == 0){
           $sLine = HAATTR . " -add $sNewType TransitionWaitLimit -integer 10\n"; 
           print TEMPFILDES $sLine; 
        }
        if($sFlag_AttrStartOptions == 0){
           $sLine = HAATTR . " -add $sNewType StartOptions -string\n";
           print TEMPFILDES $sLine;
        }
        if($sFlag_AttrStopOptions == 0){
           $sLine = HAATTR . " -add $sNewType StopOptions -string \"%-i\"\n";
           print TEMPFILDES $sLine;
        }
        if($sFlag_AttrCleanIPC == 0){
           $sLine = HAATTR . " -add $sNewType CleanIPC -boolean 0\n";
           print TEMPFILDES $sLine;
        }
	if($sFlag_AttrServiceName == 0){
           $sLine = HAATTR . "-add $sNewType ServiceName -string\n";
	   print TEMPFILDES $sLine;
	}
        if($sFlag_AttrSystemDEnvFile == 0) {
           $sLine = HAATTR . "-add $sNewType SystemDEnvFile -string\n";
           print TEMPFILDES $sLine;
        }

	if ($FlagOldTypeLatest == 1) {
          $sCommand = HACLUS . " -value ReadOnly -localclus";
          $sCmdOut = `$sCommand 2>&1`;
          chomp ( $sCmdOut );
          if ( 1 == $sCmdOut ) {
            $sCommand = HACONF . " -makerw > /dev/null 2>&1";
            system ( $sCommand );
            if ( $iExitCode = $? >> 8 ) {
	      printError("Failed to configure the cluster for read and write");
              close ( LOG );
              exit 1;
            }
            else {
              print LOG "[Success] The cluster is configured for read/write\n";
            }
          }

          if($sFlag_AttrStartOptions == 0){
            $sCommand = HAATTR . " -add $sNewType StartOptions -string";
            system ( $sCommand );
              if ( $iExitCode = $? >> 8 ) {
		printError ("Failed to add attribute StartOptions");
                close ( LOG );
                exit 1;
              }
              else {
                print LOG "[Success] Added attribute StartOptions using commands[$sCommand]\n";
              }
          }

          if($sFlag_AttrStopOptions == 0){
            $sCommand = HAATTR . " -add $sNewType StopOptions -string \"%-i\"";
            system ( $sCommand );
            if ( $iExitCode = $? >> 8 ) {
	      printError ("Failed to add attribute StopOptions");
              close ( LOG );
              exit 1;
            }
            else {
              print LOG "[Success] Added attribute StopOptions using commands[$sCommand]\n";
            }
          }
          if($sFlag_AttrCleanIPC == 0){
            $sCommand = HAATTR . " -add $sNewType CleanIPC -boolean 0";
            system ( $sCommand );
            if ( $iExitCode = $? >> 8 ) {
	      printError ("Failed to add attribute CleanIPC");              
              close ( LOG );
              exit 1;
            }
            else {
              print LOG "[Success] Added attribute CleanIPC using commands[$sCommand]\n";
            }
          }
	  if($sFlag_AttrTransitionWaitLimit == 0){
            $sCommand = HAATTR . " -add $sNewType TransitionWaitLimit -integer 10";
            system ( $sCommand );
            if ( $iExitCode = $? >> 8 ) {
              printError ("Failed to add attribute TransitionWaitLimit");
              close ( LOG );
              exit 1;
            }
            else {
              print LOG "[Success] Added attribute TransitionWaitLimit using commands[$sCommand]\n";
            }
          }

	  if($sFlag_AttrServiceName == 0){
            $sCommand = HAATTR . " -add $sNewType ServiceName -string";
            system ( $sCommand );
            if ( $iExitCode = $? >> 8 ) {
              printError ("Failed to add attribute ServiceName");
              close ( LOG );
              exit 1;
            }
            else {
              print LOG "[Success] Added attribute ServiceName using commands[$sCommand]\n";
            }
          }

          if($sFlag_AttrSystemDEnvFile == 0){
            $sCommand = HAATTR . " -add $sNewType SystemDEnvFile -string";
            system ( $sCommand );
            if ( $iExitCode = $? >> 8 ) {
              printError ("Failed to add attribute SystemDEnvFile");
              close ( LOG );
              exit 1;
            }
            else {
              print LOG "[Success] Added attribute SystemDEnvFile using commands[$sCommand]\n";
            }
          }

	  if(($sFlag_AttrStartOptions == 0) || 
             ($sFlag_AttrStopOptions == 0) ||
	     ($sFlag_AttrCleanIPC == 0) || 
	     ($sFlag_AttrTransitionWaitLimit == 0) ||
             ($sFlag_AttrServiceName == 0) ||
             ($sFlag_AttrSystemDEnvFile == 0) ) {
            $sCommand = HATYPE . " -modify $sNewType ArgList $sNewArgList";
            system ( $sCommand );
            if ( $iExitCode = $? >> 8 ) {
	      printError("Failed to modify ArgList");
              close ( LOG );
              exit 1;
            }
            else {
              print LOG "[Success] Modified ArgList using commands[$sCommand]\n";
            }
	    my $delayCount = 0;
	    while ($delayCount < 4) {
              $sCommand = HACONF . " -dump > /dev/null 2>&1";
              system ( $sCommand );
              if ( $iExitCode = $? >> 8 ) {
		sleep 5;
              }
              else {
                print LOG "[Success] Changed configuration is dumped in main.cf\n";
		last;	
              }
	      $delayCount = $delayCount + 1;
	    }
	    if ($delayCount == 4 ) {
	      printError("Failed to dump the changed configuration in main.cf");
              close ( LOG );
              exit 1;
	    }
            print "You have successfully added attributes in the agent type [$sNewType]\n";
          }
            print "For details, see the following log: [$sLogDir/MQ_update.log]\n";
            close ( LOG );
            exit 0;
	}


	close ( TEMPFILDES ) || die "Failed to close [$sTempCmdFile] after write: $!\n";
	close ( FILDES ) || die "Failed to close [$sMainCmdFile] after read: $!\n";

	$sCommand = HACLUS . " -value ReadOnly -localclus";
	$sCmdOut = `$sCommand 2>&1`;
	chomp ( $sCmdOut );

	if ( 1 == $sCmdOut ) {
		$sCommand = HACONF . " -makerw > /dev/null 2>&1";
		system ( $sCommand );
		if ( $iExitCode = $? >> 8 ) {
			printError("Failed to configure the cluster for read and write");
			close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] The cluster is configured for read/write\n";
		}
	}

	print LOG "Updating agent type file on this node\n";

	#------------------------------
	# Execute the filtered main.cmd
	#------------------------------
	if( ! $bFlagBothTypes ){
	    system ( "cd $sLogDir;./main_temp.cmd" );
		if ( $iExitCode = $? >> 8 ) {
			printError("Failed to add resource type [$sNewType]");
			close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] The new resource type is added\n";
		}
	}
	$bSuccessFlag = 0;
	#-------------------------------------------------
	# Change all the resources of Old Type to New Type
	#-------------------------------------------------
	push ( @lsArgList , "Critical", "Enabled" );
	foreach $sRes ( @lsRes ) {
		%hsResAttr = ();
		$sCommand = HARES . " -value $sRes Group -localclus";
		$sCmdOut = `$sCommand 2>&1`;
		if ( $iExitCode = $? >> 8 ) {
			printError("Not able to run the command [$sCommand]");
                        close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] Executed the command [$sCommand]\n";
		}
		$hsResAttr{ Group } = $sCmdOut ;

		foreach $sAttribute ( @lsArgList ) {
			if ( $sAttribute =~ m/State/ ) {
				next;
			}
			$sCommand = HARES . " -display $sRes -attribute $sAttribute -localclus";
			my @lsAttributeVal = `$sCommand 2>&1`;
			if ( $iExitCode = $? >> 8 ) {
				printError("Not able to run the command [$sCommand]");
                                close ( LOG );
				exit 1;
			}
			else {
				print LOG "[Success] Executed the command [$sCommand]\n";
			}
			shift @lsAttributeVal;
			my %hsSystemAttrVal = ();
			foreach ( @lsAttributeVal ) {
				/(\S+)\s+(\S+)\s+(\S+)\s+(.*)$/;
				if ( defined $4 ) {
					$hsSystemAttrVal{ $3 } = $4;
				}
				else {
					$hsSystemAttrVal{ $3 } = "";
				}
			}
			$hsResAttr{ $sAttribute } = \%hsSystemAttrVal;
		}

		#-----------------------------------------
		# Get resource dependency for the resource
		#-----------------------------------------
		$sCommand = HARES . " -dep $sRes";
		@lsDepList = ();
		$iPid = open ( CMD, "$sCommand 2>&1 | ");
		if( defined $iPid && $iPid > 1 ) {
			$sCmdOut = <CMD>;
			if ( ! ( $sCmdOut =~ /No Resource dependencies are configured/ ) ) {
				while ( <CMD> ) {
					push ( @lsDepList, $_ );
				}
			}
			print LOG "[Success] Executed the command [$sCommand] with Output[@lsDepList]\n";
		}
		else {
			printError("Not able to run the command [$sCommand]");
                        close ( LOG );
			exit 1;
		}
		$sCommand = HARES . " -delete $sRes > /dev/null 2>&1";
		system ( $sCommand );

		if ( $iExitCode = $? >> 8 ) {
			printError("Failed to delete resource [$sRes] of old type");
                        close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] Delete resource [$sRes] of old type\n";
		}
		chomp ( $hsResAttr{ Group } );
		$sCommand = HARES . " -add $sRes $sNewType $hsResAttr{ Group }" . "  >/dev/null 2>&1";
		system ( $sCommand );

		if ( $iExitCode = $? >> 8 ) {
			printError("Failed to add the resource [$sRes] of new type");
                        close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] Add resource [$sRes] of new type\n";
		}
		foreach $sAttribute ( @lsArgList ) {
			if ( $sAttribute =~ /(State|Group)/ ) {
				next;
			}

			my $href = $hsResAttr{ $sAttribute };
			my $bGlobalFlag = 0;
			foreach $sKey ( keys %$href    ) {
				$sCommand = ();
				if ( $sKey =~ /global/ && 
						( $bGlobalFlag = 1 ) &&
						$hsResAttr{ $sAttribute }{$sKey} =~ /\S+/ ) {
					chomp( $hsResAttr{ $sAttribute }{ $sKey } );
					if ( $hsResAttr{ $sAttribute }{ $sKey } =~ m/\-/ ) {
						$sCommand = HARES . " -modify $sRes $sAttribute \"\%$hsResAttr{ $sAttribute }{ $sKey }\" > /dev/null 2>&1";
					}
					elsif ( $hsResAttr{ $sAttribute }{ $sKey } =~ m/\s+/ ) {
						$sCommand = HARES . " -modify $sRes $sAttribute \"$hsResAttr{ $sAttribute }{ $sKey }\" > /dev/null 2>&1";
					}
					else {
						$sCommand = HARES . " -modify $sRes $sAttribute $hsResAttr{ $sAttribute }{ $sKey } > /dev/null 2>&1";
					}
				}
				elsif ( $hsResAttr{ $sAttribute }{ $sKey } =~ /\S+/ ) {
					chomp( $hsResAttr{ $sAttribute }{ $sKey } );
					if ( $hsResAttr{ $sAttribute }{ $sKey } =~ m/\-/ ) {
						$sCommand = HARES . " -modify $sRes $sAttribute \"\%$hsResAttr{ $sAttribute }{ $sKey }\" -sys $sKey > /dev/null 2>&1";
					}
					elsif ( $hsResAttr{ $sAttribute }{ $sKey } =~ m/\s+/ ) {
						$sCommand = HARES . " -modify $sRes $sAttribute \"$hsResAttr{ $sAttribute }{ $sKey }\" -sys $sKey > /dev/null 2>&1";
					}
					else {
						$sCommand = HARES . " -modify $sRes $sAttribute $hsResAttr{ $sAttribute }{ $sKey } -sys $sKey > /dev/null 2>&1";
					}

				}

				if ( !  $bGlobalFlag ) {
					my $sLocalCommand = HARES . " -local $sRes $sAttribute > /dev/null 2>&1";
					system ( $sLocalCommand );
					if ( $iExitCode = $? >> 8 ) {
						print LOG "[Failure] Failed to make the attribute [$sAttribute] local for resource[$sRes]\n";
						$bSuccessFlag = 1;
					}
					else {
						print LOG "[Success] Resource modification command [$sLocalCommand] executed for resource [$sRes]\n";
					}
					$bGlobalFlag = 1;
				}
				if ( defined $sCommand ) {
					system ( $sCommand );
					if ( $iExitCode = $? >> 8 ) {
						print LOG "[Failure] Failed to modify attribute [$sAttribute] resource[$sRes] with command [$sCommand]\n";
						$bSuccessFlag = 1;
					}
					else {
						print LOG "[Success] Resource modification command [$sCommand] executed for resource [$sRes]\n";
					}
				}
			}
		}


                #----------------------------------------------
                # Modifying if any overridden type level attribute
                #----------------------------------------------
 
                if ( ! open ( FILDES, $sMainCmdFile ) ) {
		     printError("Failed to open created file [$sMainCmdFile] for read");
                     close ( LOG );
                     exit 1;
                }  else {
                     print LOG "[Success] main.cmd is opened for read\n";
                }

                print LOG "Checking if there are any over-ridden attributes...\n";

                while ( $sLine = <FILDES> ) {
                      if ( $sLine =~ m/hares -override\s+\b$sRes\b\s+(\S+)/g ){
                           $sCommand = HARES . " -override $sRes $1 > /dev/null 2>&1";
                           system ( $sCommand );
                           if ( $iExitCode = $? >> 8 ) {
				printError("Failed to override attribute [$1] for resource[$sRes] with command [$sCommand]");
                           } else {
                                 print LOG "[Success] Command [$sCommand] executed for overriding attribute[$1] for resource [$sRes]\n";
                           }
                      }

                      my ($key, $value) = ();
                      my ($attr, $attrval) = ();

                      if ( $sLine =~ m/hares -modify\s+\b$sRes\b\s+(\S+)\s+(\S+)/g ){
                           $attr = $1;
                           $attrval  = $2;

                           foreach $key (keys %hsOverriddenResAttr) {

                              if($key =~ m/$sRes/){

                                  foreach $value (@{$hsOverriddenResAttr{$key}}){ 

                                      if($attr =~ /$value/){
                                         $sCommand = HARES . " -modify $sRes $attr $attrval > /dev/null 2>&1";
                                         system ( $sCommand );
                                         if ( $iExitCode = $? >> 8 ) {
                                              print LOG "[Failure] Failed to modify overridden attribute [$attr] for resource[$sRes] with command [$sCommand]\n";
                                         } else {
                                             print LOG "[Success] Command [$sCommand] executed for overriding attribute[$attr] for resource [$sRes]\n";
                                         }
                                         last;
                                      }
                                  }
                              }
                           }
                      }
                }

                close ( FILDES ) || die "Failed to close [$sMainCmdFile] after read: $!\n";

	 	#print "Configured: $sRes resource\n";

		#-------------------
		# Adding Dependancy 
		#-------------------
		if ( scalar @lsDepList ) {
			foreach my $sDep ( @lsDepList ) {
				$sDep =~ /(\S+)\s+(\S+)\s+(\S+)/;
				if ( defined $3 ) {
					$sCommand = HARES . " -link $2 $3 > /dev/null 2>&1";
					system ( $sCommand );
					if ( $iExitCode = $? >> 8 ) {
						print LOG "[Failure] Failed to link resource [$2] and resource [$3] \n";
						$bSuccessFlag = 1;
					}
					else {
						print LOG "[Success] Resource link command [$sCommand] executed for resource [$2] and [$1]\n";
					}
				}
				else {
					print LOG "[Failure] Unable to get parent and child for dependency link, for resource [$sRes]\n";
				}
			}
		}

	}

	$sCommand = HATYPE . " -delete $sOldType > /dev/null 2>&1";
	#print "\nDo you want to Delete the Agent Type $sOldType(y/n):";
	#$sConfirm = <>;
	#if ( ! ( $sConfirm =~ /^(y|Y)$/ ) ){
	#	print LOG "[INFO] Deletion of old agent type [$sOldType] from the cluster skipped..\n";
	#}
	#else {
		system ( $sCommand );

		if ( $iExitCode = $? >> 8 ) {
			print LOG "[Failure] Failed to delete old agent type [$sOldType] from the cluster\n";
			$bSuccessFlag = 1;
		}
		else {
			print LOG "[Success] Old agent type [$sOldType] deleted\n";
			#print "\nOld Agent type [$sOldType]  has been deleted \n";
		}
	#}
	#-----------------------
	# Modify new attributes 
	#-----------------------
	if ( 0 == $bSuccessFlag ) {
		#-------------------------------
		# Dump the changed configuration
		#-------------------------------
		$sCommand = HACONF . " -dump > /dev/null 2>&1";
		system ( $sCommand );

		if ( $iExitCode = $? >> 8 ) {
			printError("Failed to dump the changed configuration in main.cf");
			close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] Changed configuration is dumped in main.cf\n";
		}


		print "You have successfully replaced agent type [$sOldType] with the new agent type [$sNewType]\n";
		print "For details, see the following log: [$sLogDir/MQ_update.log]\n";
		close ( LOG );
		exit 0;
	}
	warn "The update process did not complete successfully.\n";
}
else { 	
    print LOG "VCS is currently not running.\n";

	my ( $sOldTypeFile, @lsOldArgList, @lsNewArgList, 
	     @lsDepAttr, @lsNewAttr ) = ();

	#-----------------------------
	# Open main.cmd file for Read 
	#-----------------------------
	$sMainCmdFile = "$sLogDir/main.cmd";

	if ( ! open ( MAINCMDFILDES, $sMainCmdFile ) ) {
		printError("Failed to open created file [$sMainCmdFile] for read");
		close ( LOG );
		exit 1;
	}
	else {
		print LOG "[Success] main.cmd is opened for read\n";
	}

	my @lsMainCmd = <MAINCMDFILDES>;

	#-----------------------------------------------
	# Proceed if WebSphereMQ6 type is imported 
	# or there are any resources of WebSphereMQ6
	#-----------------------------------------------
	
	if ( grep { /^hatype -add \b$sOldType\b/ } @lsMainCmd ) {
		print LOG "[INFO] $sOldType exists in main.cmd";
	}
	else {
		if ( grep { /^hatype -add \b$sNewType\b/ } @lsMainCmd ) {
			print LOG "[INFO] $sNewType exists in main.cmd";
			$sOldType = $sNewType ;
			$FlagOldTypeLatest = 1;
		}
		else {
		print LOG "[INFO] $sOldType or $sNewType doesn't exist in main.cmd.. Exiting.. ";
		close ( LOG );
		exit 0;
		}
	}

	#-------------------------------
	# Create Temporary main.cmd file
	#-------------------------------
	$sTempCmdFile = "$sLogDir/main_temp.cmd";

	if ( ! sysopen ( TEMPFILDES, $sTempCmdFile,
				O_WRONLY|O_CREAT|O_TRUNC, 0755 ) ) {
		printError("Failed to create and open file [$sTempCmdFile]: $!");
		close ( LOG );
		exit 1;
	}
	else {
		print LOG "[Success] Temprory File [$sTempCmdFile] is created and opened ".
			"for filtering Type specific data\n";
	}

	#----------------------------------------
	# Get Old ArgList Values from the 
	# Old AgentTypes.cf file.
	#----------------------------------------

	$sOldTypeFile = MAINCFLOC . "/${sOldType}Types.cf";
        if ( $^O eq 'solaris' ){
              # For VCS 5.1 or later.
              $sOldTypeFile = MAINCFLOC . "/${sOldType}Types51.cf";
              if(! -e $sOldTypeFile){
                 # For VCS 5.0
                 $sOldTypeFile = MAINCFLOC . "/${sOldType}Types50.cf";
              }
        }

	&getArgList ( $sOldTypeFile, \@lsOldArgList );
	print LOG "Old Type ArgList : " . join(', ', @lsOldArgList ) . "\n";

	#----------------------------------------
	# Get New ArgList Values from the 
	# New AgentTypes.cf file.
	#----------------------------------------

	&getArgList ( $sNewTypeFile, \@lsNewArgList );
	print LOG "New Type ArgList : " . join(', ', @lsNewArgList ) . "\n";

	#------------------------------------------------
	# Extract Deprecated and Newly added Attributes 
	#------------------------------------------------

	&getListDelta ( \@lsOldArgList, \@lsNewArgList, \@lsDepAttr );
	print LOG "Attributes To Deprecate : " . join(', ', @lsDepAttr ) . "\n";

	&getListDelta ( \@lsNewArgList, \@lsOldArgList, \@lsNewAttr );
	print LOG "Attributes To Add: " . join(', ', @lsNewAttr ) . "\n";

	#------------------
	# Modify main.cmd
	#------------------
	my ( @lsTempMainCmd, $sWL_HOME ) = ();

	foreach my $sLine ( @lsMainCmd ) {
		#--------------------------------------------
		# Update hattr and hatype. Update ArgList
		#--------------------------------------------
		if ( ( ( $sLine =~ m/\b$sOldType\b/g ) && 
					( $sLine =~ m/^\b(hatype|haattr)\b/ ) ) &&
				! ( $sLine =~ m/^hares/ ) ) {

			if ( $sLine =~ m/^hatype -add \b$sOldType\b/ ) {
				$sLine =~ s/$sOldType/$sNewType/;
			}

			if ( $sLine =~ m/^hatype -modify \b$sOldType\b (SourceFile|AgentDirectory)/ ) {
				$sLine =~ s/$sOldType/$sNewType/g;
			}
			if ( $sLine =~ m/^hatype -modify $sOldType \bArgList\b/ ) { 
				print LOG "Replacing Arglist\n";
				$sNewArgList = join (' ', @lsNewArgList); 

				$sLine =~ s/\b$sOldType\b/$sNewType/;
				$sLine =~ s/\bArgList.*/ArgList $sNewArgList/;

				print LOG "[Success] ArgList updated <-> $sLine";
			}
                        if ( $sLine =~ m/^haattr -(add|default)( -static | )\b$sOldType\b/ ) {
                                if($sLine =~ m/MonitorListener -boolean 0/){
                                   $sLine =~ s/MonitorListener -boolean 0/MonitorListener -integer 0/;
                                }
			        $sLine =~ s/$sOldType/$sNewType/;
			}else {
				$sLine =~ s/$sOldType/$sNewType/;
			}

			print LOG "[Success] haattr/hatype updated <-> $sLine";
		}

		#-----------------------------------------------
		# Change hares -add <Resoruce> <OldType> <Group>
		#-----------------------------------------------
		if ( $sLine =~ m/^hares -add \w+ $sOldType/ ) {
			$sLine =~ s/$sOldType/$sNewType/;
		}

		push ( @lsTempMainCmd, $sLine );
	}

	#-------------------------------------------------
	# Get the type and default value of new attributes
	#-------------------------------------------------
	my ( %hNewAttr, %hDefValue )= ();

	open ( FILDES, $sNewTypeFile) || die "Failed to open [$sNewTypeFile] for read: $!\n";
	my @lsNewType = <FILDES>;
	foreach my $sNewAttr (@lsNewAttr)
	{
		grep {
			s/\n//;
			s/^\s+//;
			s/^static//;
			/^(\w+)\s+$sNewAttr/ && ( $hNewAttr{$sNewAttr} = $1 );
			/$sNewAttr\s+=\s+(\S+)/ && ( $hDefValue{$sNewAttr} = $1 );
			/\)\s*$/;
		}@lsNewType;
	}

	close ( FILDES ) || die "Failed to close [$sNewTypeFile] after read: $!\n";

	#---------------------------------
	# hash to convert short type name 
	# to full name for haattr -add 
	#---------------------------------
	my %hsTypes = ( 'str' => 'string',
			'int' => 'integer',
                        'keylist' => 'keylist',
			'boolean' => 'boolean' );

	#---------------------
	# Add New Attributes
	#---------------------
	print LOG "\nAdding new attributes :\n";
	foreach my $sAttr ( keys(%hNewAttr) ){
		my ( $sType, $sLine ) = ();
		$sType = $hNewAttr{$sAttr};
		chomp $sType;
		if ( $hDefValue{$sAttr} ) {
			$sLine = "haattr -add $sNewType $sAttr -$hsTypes{ $sType } $hDefValue{$sAttr}\n";
		}
		else {
			$sLine = "haattr -add $sNewType $sAttr -$hsTypes{ $sType }\n";
		}

		print LOG "$sLine";
		push ( @lsTempMainCmd, $sLine );
	}

	###-----------------------------------------
	# haattr -delete option is not supported 
	# in hacf -cmdtocf so search and replace
	###-----------------------------------------

	#---------------------
	# Deprecate Attributes
	#---------------------
	print LOG "\nDepricating Attribute:\n";
	foreach my $sAttr ( @lsDepAttr ){
		grep { s/(hares|haattr)\s+(-add|-modify)\s+\S+\s+$sAttr\s+.*//g } @lsTempMainCmd;
		print LOG "[Success] $sAttr Attribute depricated\n";
	}

	#--------------------------
	# Write into main_temp.cmd
	#--------------------------
	print TEMPFILDES @lsTempMainCmd;

	# Copy main_temp.cmd to /etc/VRTSvcs/conf/config/main.cmd 

	$sCommand = "/bin/cp -f $sLogDir/main_temp.cmd " . MAINCFLOC . "/main.cmd > /dev/null 2>&1";
	system ( $sCommand );
	if ( $iExitCode = $? >> 8 ) {
		printError("Not able to copy main_temp.cmd to [/etc/VRTSvcs/conf/config/main.cmd]");
		close(LOG);
		exit 1;
	}
	else {
		print LOG "[Success] Executed the command [$sCommand]\n";
	}

	print LOG "[Success] Sucessfully modified main.cmd for $sOldType to $sNewType\n";

	close ( TEMPFILDES ) || die "Failed to close [$sTempCmdFile] after read: $!\n";
	close ( MAINCMDFILDES ) || die "Failed to close [$sMainCmdFile] after read: $!\n";

	#-------------------------------------------------------
	# if main.cmd created succesfully, convert to main.cf 
	# using hacf -cmdtocf command
	#-------------------------------------------------------
	if ( -e MAINCFLOC . "/main.cmd" ) {
		$sCommand = HACF . " -cmdtocf " . MAINCFLOC . " > /dev/null 2>&1";
		print LOG "Running command [ $sCommand ] to create modified main.cf\n";
		system ( $sCommand );
		if( $iExitCode = $? >> 8 ) {
			printError("Failed to create main.cf from main.cmd file. ExitCode [$iExitCode]");
			close ( LOG );
			exit 1;
		}  
		else {
			print LOG "[Success] main.cf file created successfully.\n";
			print "You have successfully replaced agent type [$sOldType] with the new agent type [$sNewType]\n";
			print "For details, see the following log: [$sLogDir/MQ_update.log]\n";
			close ( LOG );
			exit 0;
		}
	}
	else {
		print LOG "[Failed] main.cmd doesn't exists. Bailing out\n";
		close ( LOG );
		exit 1;
	}
}

#------------------------------------------------
# Subroutine to fetch ArgList from Types.cf File
#------------------------------------------------
sub getArgList {
	my ( $sAgentTypesFile, 
	     $rlsArgList )      = @_;

	#----------------------------------------
	# Get ArgList from the AgentTypes.cf file
	#----------------------------------------
	if ( -e $sAgentTypesFile ) {
		if ( ! open ( TYPEFILDES, $sAgentTypesFile ) ) {
			printError("Failed to open Agent type file [ $sAgentTypesFile ] for read");
			close ( LOG );
			exit 1;
		}
		else {
			print LOG "[Success] AgentTypes File [ $sAgentTypesFile ] is opened for read\n";
		}
	}
	else {
		printError("AgentTypes file [ $sAgentTypesFile ] does not existi, Re-install the latest agent package");
		close ( LOG );
		exit 1;
	}

	while (<TYPEFILDES>) {
		s/\n//;
		s/^\s+//;
		/^static\s+str\s+ArgList/ && ( @$rlsArgList = $_ =~ /[,{] (\w+)/g );
			/\)\s*$/;
	}

	close ( TYPEFILDES ) || die "Failed to close [ $sAgentTypesFile ] after read: $!\n";
	print LOG "[Success] Get the ArgList From [ $sAgentTypesFile ] File\n";
}

#----------------------------------------------
# Subroutine to check if VCS is running or not
#----------------------------------------------
sub getVCSStatus {
        my $sCmd = HASYS . " -list -localclus";
        my @lsCmdOut = `$sCmd 2>&1`;

        $bVCSStatus = 1; # Assume vcs is currently running 

        if ( grep { /Cannot connect to VCS engine/ } @lsCmdOut ) {
                print LOG "VCS is not running. Following Offline Procedure.\n";
                $bVCSStatus = 0;
        }
        else {
                print LOG "VCS is running. Following Online Procedure.\n";
        }

	return $bVCSStatus;
}


#------------------------------------------------
# Subroutine to subtract two lists
#------------------------------------------------
sub getListDelta {
	my ( $rlsFirst, 
	     $rlsSecond,
	     $rlsDelta ) = @_;

	my %hCnt = map { $_ => 1 }@$rlsSecond;
	@$rlsDelta = grep { not $hCnt{$_}} @$rlsFirst;
}

sub printError {
	my ( $sErrorString ) = @_;
	  warn "$sErrorString\n";
 	  print LOG "[Failure] $sErrorString\n";
	  print "For more details check the log file [$sLogDir/MQ_update.log]\n";
}

warn "For more details check the log file [$sLogDir/MQ_update.log] $sRestoreMsg\n";
exit 1;
